/********************************************************/
/*                                                      */
/*               OO7 Benchmark                          */
/*                                                      */
/*              COPYRIGHT (C) 1993                      */
/*                                                      */
/*                Michael J. Carey 		        */
/*                David J. DeWitt 		        */
/*                Jeffrey Naughton 		        */
/*               Madison, WI U.S.A.                     */
/*                                                      */
/*	         ALL RIGHTS RESERVED                    */
/*                                                      */
/********************************************************/

////////////////////////////////////////////////////////////////////////////
//
// AtomicPart Methods
//
////////////////////////////////////////////////////////////////////////////

#include <libc.h>
#include <sys/time.h>
#include <sys/resource.h>
#include <stdio.h>

#include "oo7.h"
#include "globals.h"
#include "VarParams.h"
#include "GenParams.h"

extern o_4b	debugMode;
extern char*	types[NumTypes];

Implement PClassObj<AtomicPart>;

////////////////////////////////////////////////////////////////////////////
//
// AtomicPart Constructor
//
////////////////////////////////////////////////////////////////////////////

AtomicPart::AtomicPart(o_4b ptId, Link<CompositePart> cp) :

//    id(ptId), partOf(cp), to(3), from(3)
    id(ptId), partOf(cp), to(), from()
{
    if (debugMode) {
        printf("AtomicPart::AtomicPart(ptId = %d)\n", ptId);
    }

    // initialize internal state of new part
    o_4b	typeNo = (o_4b) (random() % NumTypes);

    strncpy(type, types[typeNo], TypeSize);
    buildDate = MinAtomicDate +
        (o_4b)(random() % (MaxAtomicDate-MinAtomicDate+1));
    x = (o_4b)(random() % XYRange);
    y = (o_4b)(random() % XYRange);
    // fill in a random document id (for query 9)
    docId = (o_4b)random() % TotalCompParts + 1;
}

// destructor
AtomicPart::~AtomicPart()
{
    o_word	i;
    o_4b 	size;

    size = to.size();
    for (i = 0; i < size; i++) {
	delete (Connection*)to[i];
    }
    from.release();
    to.release();
}

////////////////////////////////////////////////////////////////////////////
//
// AtomicPart swapXY Method for use in update traversals
//
////////////////////////////////////////////////////////////////////////////

void AtomicPart::swapXY()
{
    if (debugMode) {
        printf("                    AtomicPart::swapXY(ptId = %d, x = %d, y = %d)\n", id,x,y);
    }

    dirty();

    // exchange X and Y values
    o_4b	tmp = x;

    x = y;
    y = tmp;

    if (debugMode) {
        printf("                    [did swap, so x = %d, y = %d]\n", x, y);
    }
}


////////////////////////////////////////////////////////////////////////////
//
// AtomicPart toggleDate Method for use in update traversals
//
////////////////////////////////////////////////////////////////////////////

void AtomicPart::toggleDate()
{
    if (debugMode) {
        printf("                    AtomicPart::toggleDate(ptId = %d, buildDate = %d)\n",
	       id, buildDate);
    }

    // put the change into the DB

    // increment build date if odd, decrement it if even

    dirty();

    if (buildDate % 2) {
	// odd case
	buildDate++;
    } else {
	// even case
	buildDate--;
    }

    if (debugMode) {
        printf("                    [did toggle, so buildDate = %d]\n", buildDate);
    }
}


////////////////////////////////////////////////////////////////////////////
//
// AtomicPart DoNothing Method 
//
////////////////////////////////////////////////////////////////////////////

o_4b RealWork; // set to one to make DoNothing do work.
o_4b WorkAmount; // amount of work to do

void AtomicPart::DoNothing()
{
    if (id < 0) 
        printf("DoNothing - negative id\n");


    if (debugMode) {
        printf("==> DoNothing(x = %d, y = %d, type = %s)\n", x, y, type);
    }

    if (RealWork) {
	struct timeval tv;
	struct timezone tz;
	o_4b i;

	for (i = 0; i < WorkAmount; i++) {
	   gettimeofday(&tv, &tz);
	}
    }
}
