/********************************************************/
/*                                                      */
/*               OO7 Benchmark                          */
/*                                                      */
/*              COPYRIGHT (C) 1993                      */
/*                                                      */
/*                Michael J. Carey 		        */
/*                David J. DeWitt 		        */
/*                Jeffrey Naughton 		        */
/*               Madison, WI U.S.A.                     */
/*                                                      */
/*	         ALL RIGHTS RESERVED                    */
/*                                                      */
/********************************************************/

#include <libc.h>
#include <stdio.h>
#include <sys/time.h>
#include <sys/resource.h>
#include <strings.h>
#include <stdlib.h>

#include "oo7.h"
#include "globals.h"
#include "GenParams.h"
#include "VarParams.h"
#include "stats.h"
#include "stats_server.h"
#include "baidlist.h"
#include "Index.h"
#include <cxxcls/vdiction.h>

extern "C" char* getenv(const char*);

// This is a gendb hacked to generate the large db in pieces.
// It takes two parameters.  The starting composite partid and
// the ending composite part id.  Modules and assemblies are generated
// only when the second parameter equals the total number of
// composite parts

//////////////////////////////////////////////////////////////////
//
// Global Variables, etc., for Benchmark Database Generation
//
//////////////////////////////////////////////////////////////////


struct rusage startUsage, endUsage;
struct timeval startWallTime;
struct timeval endWallTime;
struct timezone ignoreTimeZone;

extern SetParams(char* configFileName);
extern void InitGlobals();
extern void EstablishGlobals();
extern void createDummy(o_4b	cnt);

extern double ComputeWallClockTime(struct timeval *startWallTime, 
	           struct timeval *endWallTime);

extern double ComputeUserTime(struct rusage *startUsage, 
	           struct rusage *endUsage);

extern double ComputeSystemTime(struct rusage *startUsage, 
	           struct rusage *endUsage);

o_4b	nextAtomicId = 1;
o_4b	nextCompositeId = 1;
o_4b	nextComplexAssemblyId = 1;
o_4b	nextBaseAssemblyId = 1;
o_4b	nextModuleId = 1;
o_4b	debugMode = TRUE;

char    *types[NumTypes] = {
	"type000", "type001", "type002", "type003", "type004",
	"type005", "type006", "type007", "type008", "type009"
    };

// data structures to keep track of which composite parts each base
// assemblies uses.  This is done so that we can generate the base
// assemblies first and then the composite parts.  Why bother?
// In particular, if you did the composite parts first and the base
// assemblies, there would be no need for these data structures.
// This second approach worked fine for the small and medium databases
// but not for the large as the program seeked all over the disk.

BAIdList* Private_cp;
BAIdList* Shared_cp;

// JGM 93/10/15 Added space between '>' tokens to avoid cfront 3.0
// warnings

extern Link<VEIDictionary<o_4b, Document> >	documentIndex;

//////////////////////////////////////////////////////////////////
//
// Do Benchmark Data Generation
//
//////////////////////////////////////////////////////////////////

void gendb(o_4b	startCompId, o_4b endCompId)
{
    o_4b	id;

    // First generate the desired number of modules
    if (startCompId == 1) 
    { 
	// first run, generate modules
    	while (nextModuleId <= TotalModules) 
	{
	    Link<Module>	module;

	    id = nextModuleId++;
	    printf("about to generate module %d\n",id);
	    fflush(stdout);
	    module = new Persistent Module(id);
	}
	// generate document index
	documentIndex = new Persistent VEIDictionary<o_4b, Document>
			(NumCompPerModule*TotalModules);

	Index	docIndex("document index", documentIndex);
    }

    printf("Made modules, now on to composite parts.\n");

    // now create a batch of composite parts.
    nextCompositeId = startCompId;
    nextAtomicId = (startCompId-1)*NumAtomicPerComp + 1;
    while (nextCompositeId <= endCompId) 
    {
	Link<CompositePart>	compositePart;

	id = nextCompositeId++;
	compositePart = new Persistent CompositePart(id);

	if ((nextCompositeId % 100) == 0) {
	    printf("Made composite part %d.\n", nextCompositeId);
            fflush(stdout); 
// JGM 93/11/4 Changed next line to use the normal C++ commit.
	    // ::dom->xact(O_COMMIT_WITH_CLEAN_CODS, "creation");
	    ::dom->commit();
        }
    }
    printf("made composite parts %d to %d.\n", startCompId, endCompId);
    gettimeofday(&endWallTime, &ignoreTimeZone);       
    printf("Wall-Clock time to generate database without index: %f seconds.\n", 
	   ComputeWallClockTime(&startWallTime, &endWallTime));

    // Print out some useful information about what happened
    printf("=== DONE CREATING DATABASE, TOTALS WERE ===\n");
    printf("    # atomic parts\t\t%d\n", nextAtomicId-1);
    printf("    # composite parts\t\t%d\n", nextCompositeId-1);
    printf("    # complex assemblies\t%d\n", nextComplexAssemblyId-1);
    printf("    # base assemblies\t\t%d\n", nextBaseAssemblyId-1);
    printf("    # modules\t\t\t%d\n", nextModuleId-1);

    // create all the indices needed
    PClassObject<AtomicPart>::Object().createindex("AtomicPart::buildDate",
						   O_IT_BTREE);
    PClassObject<AtomicPart>::Object().createindex("AtomicPart::id", O_IT_HASH);
    PClassObject<CompositePart>::Object().createindex("CompositePart::id",
						      O_IT_HASH);
    PClassObject<Document>::Object().createindex("Document::title", O_IT_HASH);
    PClassObject<Module>::Object().createindex("Module::id", O_IT_HASH);
    PClassObject<BaseAssembly>::Object().createindex("Assembly::id", O_IT_HASH);
    printf("made indices.\n");
    ::dom->commit();

    // compute and report wall clock time
    gettimeofday(&endWallTime, &ignoreTimeZone);       
    getrusage(RUSAGE_SELF, &endUsage);

    //////////////////////////////////////////////////////////////////
    //
    // Shutdown DB
    //
    //////////////////////////////////////////////////////////////////

    printf("Wall-Clock time to generate database: %f seconds.\n", 
	   ComputeWallClockTime(&startWallTime, &endWallTime));
    fprintf(stdout, "(CPU: %f seconds.)\n", 
	    ComputeUserTime(&startUsage, &endUsage) +
	    ComputeSystemTime(&startUsage, &endUsage));

    ::dom->exit(0);
}

#define DBNAME	"oo7db"
#define SESSION "GenDB"

main(o_4b argc, char** argv)
{
    ProcessStats	totalTime;
    ServerStats		totalSrvTime;
    char*		purgeVar;

    if (argc < 4) 
    {
	  fprintf(stderr, 
	      "Usage: %s <configFileName> <startCompId> <endCompId> [-d]\n", argv[0]);
	  exit(1);
    }
    // See if debug mode is desired
    if ((argc == 5) && (strcmp(argv[4], "-d") == 0)) debugMode = TRUE;
    else  debugMode = FALSE;
    SetParams(argv[1]);

    // allocate space for BAlists
    Shared_cp = new BAIdList[TotalCompParts+1];
    Private_cp = new BAIdList[TotalCompParts+1];

    // get starting composite partID.
    o_4b	startCompID  = atoi(argv[2]);
    o_4b	endCompID  = atoi(argv[3]);

    printf("generating composite parts %d to %d\n", startCompID, endCompID);
    fflush(stdout);

    // get wall clock time
    gettimeofday(&startWallTime, &ignoreTimeZone);

    // get starting usage values.
    getrusage(RUSAGE_SELF, &startUsage);

    totalTime.Start();
    totalSrvTime.Start();

    // set random number generator
    srandom(1);

    ::dom = new PDOM;
    ::dom->beginsession(DBNAME,SESSION);

    // generate the database
    gendb(startCompID, endCompID); 

    totalTime.Stop();
    totalSrvTime.Stop();
    fprintf(stdout, "Total stats (client,server):\n");
    totalTime.PrintStatsHeader(stdout);
    totalTime.PrintStats(stdout, "TotalCli");
    totalSrvTime.PrintStats(stdout, "TotalSrv");

    // create 60,000 1000 byte dummy objects
    purgeVar = getenv("PURGE");
    if (purgeVar != NULL) {
	if (atoi(purgeVar) == 1) {
	    printf("creating 60000 dummy 1,000 byte objects\n");
	    createDummy(60000);
	}
	else printf("not creating purge collection\n");
    }
    else printf("not creating purge collection\n");
    ::dom->exit(0);
}
